@@ -65,7 +65,7 @@ class TagContinuous(CUDAEnvironmentContext):
         step_reward_for_runner=0.0,
         end_of_game_reward_for_runner=1.0,
         runner_exits_game_after_tagged=True,
-        use_cuda=False,
+        env_backend="cpu",
     ):
         """
         Args:
@@ -79,7 +79,7 @@ class TagContinuous(CUDAEnvironmentContext):
                 agents]. Defaults to None.
             starting_location_y ([ndarray], optional): [starting y locations of the
             agents]. Defaults to None.
-            starting_directions ([ndarray], optional): [starting orientations
+            starting_directions ([ndarray], optional): starting orientations
                 in [0, 2*pi]. Defaults to None.
             seed ([type], optional): [seeding parameter]. Defaults to None.
             max_speed (float, optional): [max speed of the agents]. Defaults to 1.0
@@ -119,9 +119,9 @@ class TagContinuous(CUDAEnvironmentContext):
             runner_exits_game_after_tagged (bool, optional): [boolean indicating
                 whether runners exit the game after getting tagged or can remain in and
                 continue to get tagged]. Defaults to True.
-            use_cuda (bool, optional): [boolean to indicate whether to use the CPU
-                or the GPU. (cuda) for stepping through the environment].
-                Defaults to False.
+            env_backend (string, optional): [indicate whether to use the CPU
+                or the GPU (either pycuda or numba) for stepping through the environment].
+                Defaults to "cpu".
         """
         super().__init__()
 
@@ -232,7 +232,7 @@ class TagContinuous(CUDAEnvironmentContext):
         # Add action 0 - this will be the no-op, or 0 turn
         self.turn_actions = np.insert(self.turn_actions, 0, 0).astype(self.float_dtype)
 
-        # Tagger and runner agent skill levels
+        # Tagger and runner agent skill levels.
         # Skill levels multiply on top of the acceleration levels
         self.skill_levels = [
             self.agent_type[agent_id] * self.float_dtype(skill_level_tagger)
@@ -248,7 +248,7 @@ class TagContinuous(CUDAEnvironmentContext):
         self.global_state = None
 
         # Defining observation and action spaces
-        self.observation_space = None  # This will be set via the env_wrapper (in utils)
+        self.observation_space = None  # Note: this will be set via the env_wrapper
         self.action_space = {
             agent_id: spaces.MultiDiscrete(
                 (len(self.acceleration_actions), len(self.turn_actions))
@@ -256,9 +256,9 @@ class TagContinuous(CUDAEnvironmentContext):
             for agent_id in range(self.num_agents)
         }
         # Used in generate_observation()
-        # If use_full_observation is True, then all the agents will have info of
-        # all the other agents, otherwise, each each agent will only have info of
-        # its K nearest other agents (K = num_other_agents_observed)
+        # When use_full_observation is True, then all the agents will have info of
+        # all the other agents, otherwise, each agent will only have info of
+        # its k-nearest agents (k = num_other_agents_observed)
         self.use_full_observation = use_full_observation
         self.init_obs = None  # Will be set later in generate_observation()
 
@@ -300,9 +300,7 @@ class TagContinuous(CUDAEnvironmentContext):
         self.still_in_the_game = None
 
         # These will also be set via the env_wrapper
-        # use_cuda will be set to True (by the env_wrapper), if needed
-        # to be simulated on the GPU
-        self.use_cuda = use_cuda
+        self.env_backend = env_backend
 
         # Copy runners dict for applying at reset
         self.runners_at_reset = copy.deepcopy(self.runners)
@@ -422,10 +420,10 @@ class TagContinuous(CUDAEnvironmentContext):
             ** 2
         ).astype(self.float_dtype)
 
-    def K_nearest_neighbors(self, agent_id, K):
+    def k_nearest_neighbors(self, agent_id, k):
         """
-        Note: 'K_nearest_neighbors' is only used when running on CPU step() only.
-        When using the CUDA step function, this Python method (K_nearest_neighbors)
+        Note: 'k_nearest_neighbors' is only used when running on CPU step() only.
+        When using the CUDA step function, this Python method (k_nearest_neighbors)
         is also part of the step() function!
         """
         agent_ids_and_distances = []
@@ -435,13 +433,13 @@ class TagContinuous(CUDAEnvironmentContext):
                 agent_ids_and_distances += [
                     (ag_id, self.compute_distance(agent_id, ag_id))
                 ]
-        K_nearest_neighbor_ids_and_distances = heapq.nsmallest(
-            K, agent_ids_and_distances, key=lambda x: x[1]
+        k_nearest_neighbor_ids_and_distances = heapq.nsmallest(
+            k, agent_ids_and_distances, key=lambda x: x[1]
         )
 
         return [
             item[0]
-            for item in K_nearest_neighbor_ids_and_distances[
+            for item in k_nearest_neighbor_ids_and_distances[
                 : self.num_other_agents_observed
             ]
         ]
@@ -552,10 +550,10 @@ class TagContinuous(CUDAEnvironmentContext):
 
                 # Set obs for agents still in the game
                 if self.still_in_the_game[agent_id]:
-                    nearest_neighbor_ids = self.K_nearest_neighbors(
-                        agent_id, K=self.num_other_agents_observed
+                    nearest_neighbor_ids = self.k_nearest_neighbors(
+                        agent_id, k=self.num_other_agents_observed
                     )
-                    # For the case when the number of nearest agent ids is fewer
+                    # For the case when the number of remaining agent ids is fewer
                     # than self.num_other_agents_observed (because agents have exited
                     # the game), we also need to pad obs wih zeros
                     obs_global_states = np.hstack(
@@ -732,7 +730,7 @@ class TagContinuous(CUDAEnvironmentContext):
         )
         data_dict.add_data(
             name="neighbor_distances",
-            data=np.zeros((self.num_agents, self.num_agents - 1), dtype=np.int32),
+            data=np.zeros((self.num_agents, self.num_agents - 1), dtype=np.float32),
             save_copy_and_apply_at_reset=True,
         )
         data_dict.add_data(
@@ -783,7 +781,7 @@ class TagContinuous(CUDAEnvironmentContext):
             key=_ACC, value=self.starting_accelerations, t=self.timestep
         )
 
-        # Array to keep track of who all are still in the game
+        # Array to keep track of the agents that are still in play
         self.still_in_the_game = np.ones(self.num_agents, dtype=self.int_dtype)
 
         # Initialize global state for "still_in_the_game" to all ones
@@ -805,7 +803,7 @@ class TagContinuous(CUDAEnvironmentContext):
         Env step() - The GPU version calls the corresponding CUDA kernels
         """
         self.timestep += 1
-        if self.use_cuda:
+        if not self.env_backend == "cpu":
             # CUDA version of step()
             # This subsumes update_state(), generate_observation(),
             # and compute_reward()
@@ -845,11 +843,17 @@ class TagContinuous(CUDAEnvironmentContext):
                 ("n_agents", "meta"),
                 ("episode_length", "meta"),
             ]
-            self.cuda_step(
-                *self.cuda_step_function_feed(args),
-                block=self.cuda_function_manager.block,
-                grid=self.cuda_function_manager.grid,
-            )
+
+            if self.env_backend == "pycuda":
+                self.cuda_step(
+                    *self.cuda_step_function_feed(args),
+                    block=self.cuda_function_manager.block,
+                    grid=self.cuda_function_manager.grid,
+                )
+            elif self.env_backend == "numba":
+                self.cuda_step[
+                    self.cuda_function_manager.grid, self.cuda_function_manager.block
+                ](*self.cuda_step_function_feed(args))
             result = None  # do not return anything
         else:
             assert isinstance(actions, dict)
@@ -873,7 +877,7 @@ class TagContinuous(CUDAEnvironmentContext):
 
             # Update state and generate observation
             self.update_state(delta_accelerations, delta_turns)
-            if not self.use_cuda:
+            if self.env_backend == "cpu":
                 obs = self.generate_observation()
 
             # Compute rewards and done
