@@ -231,16 +231,6 @@ class WarpDriveModule(LightningModule):
         # upon resetting environments for the very first time.
         self.cuda_envs.reset_all_envs()
 
-        # Create and push data placeholders to the device
-        create_and_push_data_placeholders(
-            self.cuda_envs,
-            self.policy_tag_to_agent_id_map,
-            self.create_separate_placeholders_for_each_policy,
-            self.obs_dim_corresponding_to_num_agents,
-            self.training_batch_size_per_env,
-            push_data_batch_placeholders=False,
-        )
-
         if env_wrapper.env_backend == "pycuda":
             from warp_drive.managers.pycuda_managers.pycuda_function_manager import (
                 PyCUDASampler,
@@ -258,20 +248,16 @@ class WarpDriveModule(LightningModule):
                 self.cuda_envs.cuda_function_manager
             )
 
-        # Register action placeholders
-        if self.create_separate_placeholders_for_each_policy:
-            for policy in self.policies:
-                sample_agent_id = self.policy_tag_to_agent_id_map[policy][0]
-                action_space = self.cuda_envs.env.action_space[sample_agent_id]
-
-                self._register_actions(action_space, suffix=f"_{policy}")
-        else:
-            sample_policy = self.policies[0]
-            sample_agent_id = self.policy_tag_to_agent_id_map[sample_policy][0]
-            action_space = self.cuda_envs.env.action_space[sample_agent_id]
-
-            self._register_actions(action_space)
-
+        # Create and push data placeholders to the device
+        create_and_push_data_placeholders(
+            env_wrapper=self.cuda_envs,
+            action_sampler=self.cuda_sample_controller,
+            policy_tag_to_agent_id_map=self.policy_tag_to_agent_id_map,
+            create_separate_placeholders_for_each_policy=self.create_separate_placeholders_for_each_policy,
+            obs_dim_corresponding_to_num_agents=self.obs_dim_corresponding_to_num_agents,
+            training_batch_size_per_env=self.training_batch_size_per_env,
+            push_data_batch_placeholders=False, # we use lightning to batch
+        )
         # Seeding
         seed = self.config["trainer"].get("seed", np.int32(time.time()))
         seed_everything(seed)
@@ -395,11 +381,10 @@ class WarpDriveModule(LightningModule):
         Perform the policy evaluation (forward pass through the models)
         and compute action probabilities
         """
+        assert isinstance(batch_index, int)
         probabilities = {}
         for policy in self.policies:
-            probabilities[policy], _ = self.models[policy](
-                batch_index=batch_index, batch_size=self.training_batch_size_per_env
-            )
+            probabilities[policy], _ = self.models[policy](batch_index=batch_index)
 
         # Combine probabilities across policies if there are multiple policies,
         # yet they share the same action placeholders.
@@ -420,123 +405,89 @@ class WarpDriveModule(LightningModule):
             first_policy = list(probabilities.keys())[0]
             num_action_types = num_action_types[first_policy]
 
-            first_action_idx = 0
-            num_envs = probabilities[first_policy][first_action_idx].shape[0]
+            first_action_type_id = 0
+            num_envs = probabilities[first_policy][first_action_type_id].shape[0]
             num_agents = self.cuda_envs.env.num_agents
 
             combined_probabilities = [None for _ in range(num_action_types)]
-            for action_type in range(num_action_types):
-                action_dim = probabilities[first_policy][action_type].shape[-1]
-                combined_probabilities[action_type] = torch.zeros(
+            for action_type_id in range(num_action_types):
+                action_dim = probabilities[first_policy][action_type_id].shape[-1]
+                combined_probabilities[action_type_id] = torch.zeros(
                     (num_envs, num_agents, action_dim)
                 ).cuda()
 
             # Combine the probabilities across policies
-            for action_idx in range(num_action_types):
+            for action_type_id in range(num_action_types):
                 for policy, prob_values in probabilities.items():
                     agent_to_id_mapping = self.policy_tag_to_agent_id_map[policy]
-                    combined_probabilities[action_idx][
+                    combined_probabilities[action_type_id][
                         :, agent_to_id_mapping
-                    ] = prob_values[action_idx]
+                    ] = prob_values[action_type_id]
 
             probabilities = {_COMBINED: combined_probabilities}
 
         return probabilities
 
-    def _register_actions(self, action_space, suffix=""):
-        if isinstance(action_space, Discrete):
-            # Single action
-            action_dim = [action_space.n]
-        elif isinstance(action_space, MultiDiscrete):
-            # Multiple actions
-            action_dim = action_space.nvec
-        else:
-            raise NotImplementedError(
-                "Only 'Discrete' or 'MultiDiscrete' type action spaces are supported!"
-            )
-        if len(action_dim) == 1:
-            self.cuda_sample_controller.register_actions(
-                self.cuda_envs.cuda_data_manager,
-                action_name=_ACTIONS + suffix,
-                num_actions=action_dim[0],
-            )
-        else:
-            for action_idx, _ in enumerate(action_dim):
-                self.cuda_sample_controller.register_actions(
-                    self.cuda_envs.cuda_data_manager,
-                    action_name=f"{_ACTIONS}_{action_idx}" + suffix,
-                    num_actions=action_dim[action_idx],
-                )
-
-    def _sample_actions(self, probabilities):
+    def _sample_actions(self, probabilities, batch_index=0):
         """
         Sample action probabilities (and push the sampled actions to the device).
         """
+        assert isinstance(batch_index, int)
         if self.create_separate_placeholders_for_each_policy:
             for policy in self.policies:
-                suffix = f"_{policy}"
-                self._sample_actions_helper(probabilities[policy], suffix=suffix)
+                # Sample each individual policy
+                policy_suffix = f"_{policy}"
+                self._sample_actions_helper(
+                    probabilities[policy], policy_suffix=policy_suffix
+                )
         else:
             assert len(probabilities) == 1
             policy = list(probabilities.keys())[0]
+            # sample a single or a combined policy
             self._sample_actions_helper(probabilities[policy])
 
-    def _sample_actions_helper(self, probabilities, suffix=""):
-
+    def _sample_actions_helper(self, probabilities, policy_suffix=""):
+        # Sample actions with policy_suffix tag
         num_action_types = len(probabilities)
 
         if num_action_types == 1:
-            action_name = _ACTIONS + suffix
+            action_name = _ACTIONS + policy_suffix
             self.cuda_sample_controller.sample(
                 self.cuda_envs.cuda_data_manager, probabilities[0], action_name
             )
-            # Push actions to the batch of actions
-            actions = self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
-                action_name
-            )
-            self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
-                name=f"{_ACTIONS}" + suffix
-            )[:, :, num_action_types - 1] = actions
-
         else:
-            for action_idx, probs in enumerate(probabilities):
-                action_name = f"{_ACTIONS}_{action_idx}" + suffix
+            for action_type_id, probs in enumerate(probabilities):
+                action_name = f"{_ACTIONS}_{action_type_id}" + policy_suffix
                 self.cuda_sample_controller.sample(
                     self.cuda_envs.cuda_data_manager, probs, action_name
                 )
-                # Push (indexed) actions to 'actions' and 'actions_batch'
+                # Push (indexed) actions to 'actions'
                 actions = self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
                     action_name
                 )
                 self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
-                    name=_ACTIONS + suffix
-                )[:, :, action_idx] = actions
-                self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
-                    name=f"{_ACTIONS}" + suffix
-                )[:, :, action_idx] = actions
+                    name=_ACTIONS + policy_suffix
+                )[:, :, action_type_id] = actions[:, :, 0]
 
-    def _bookkeep_rewards_and_done_flags(self):
+    def _bookkeep_rewards_and_done_flags(self, batch_index):
         """
         Push rewards and done flags to the corresponding batched versions.
         Also, update the episodic reward
         """
-        # Push done flags to done_flags_batch
+        assert isinstance(batch_index, int)
+
         done_flags = (
             self.cuda_envs.cuda_data_manager.data_on_device_via_torch("_done_") > 0
         )
 
         done_env_ids = done_flags.nonzero()
 
-        # Push rewards to rewards_batch and update the episodic rewards
+        # update the episodic rewards
         if self.create_separate_placeholders_for_each_policy:
             for policy in self.policies:
                 rewards = self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
                     f"{_REWARDS}_{policy}"
                 )
-                self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
-                    name=f"{_REWARDS}_{policy}"
-                )[:] = rewards
-
                 # Update the episodic rewards
                 self._update_episodic_rewards(rewards, done_env_ids, policy)
 
@@ -544,7 +495,6 @@ class WarpDriveModule(LightningModule):
             rewards = self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
                 _REWARDS
             )
-
             # Update the episodic rewards
             # (sum of individual step rewards over an episode)
             for policy in self.policies:
@@ -728,7 +678,7 @@ class WarpDriveModule(LightningModule):
         # Sample actions using the computed probabilities
         # and push to the batch of actions
         start_event.record()
-        self._sample_actions(probabilities)
+        self._sample_actions(probabilities, batch_index=batch_index)
         end_event.record()
         torch.cuda.synchronize()
 
@@ -737,7 +687,7 @@ class WarpDriveModule(LightningModule):
         self.cuda_envs.step_all_envs()
 
         # Bookkeeping rewards and done flags
-        done_flags = self._bookkeep_rewards_and_done_flags()
+        done_flags = self._bookkeep_rewards_and_done_flags(batch_index=batch_index)
 
         # Reset all the environments that are in done state.
         if done_flags.any():
@@ -779,7 +729,7 @@ class WarpDriveModule(LightningModule):
         for policy in self.policies_to_train:
             if self.create_separate_placeholders_for_each_policy:
                 actions = self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
-                    f"{_ACTIONS}{policy}"
+                    f"{_ACTIONS}_{policy}"
                 )
                 rewards = self.cuda_envs.cuda_data_manager.data_on_device_via_torch(
                     f"{_REWARDS}_{policy}"
@@ -804,6 +754,7 @@ class WarpDriveModule(LightningModule):
             )
         return training_batch
 
+
     # APIs to integrate with Pytorch Lightning
     # ----------------------------------------
     def train_dataloader(self):
@@ -1007,15 +958,15 @@ class PerfStatsCallback(Callback):
             print(f"{k:40}: {v:10.2f}")
         print("\n")
 
-    # Pytorch Lightning hooks
-    def on_batch_start(self, trainer=None, pl_module=None):
+    # Pytorch Lightning hooks (>= v1.8)
+    def on_train_batch_start(self, trainer=None, pl_module=None, batch=None, batch_idx=None):
         assert trainer is not None
         assert pl_module is not None
         self.iters += 1
         self.steps = self.iters * self.batch_size
         self.start_event_batch.record()
 
-    def on_batch_end(self, trainer=None, pl_module=None):
+    def on_train_batch_end(self, trainer=None, pl_module=None, outputs=None, batch=None, batch_idx=None):
         assert trainer is not None
         assert pl_module is not None
         self.end_event_batch.record()
